@@ -41,13 +41,13 @@ Settings config = {
     .modi                   = "run,ssh",
 #endif
     /** Border width around the window. */
-    .menu_bw                =                                   1,
+    .menu_bw                = 1,
     /** The width of the switcher. (0100 in % > 100 in pixels) */
-    .menu_width             =                                  50,
+    .menu_width             = 50,
     /** Maximum number of options to show. */
-    .menu_lines             =                                  15,
+    .menu_lines             = 15,
     /** Number of columns */
-    .menu_columns           =                                   1,
+    .menu_columns           = 1,
     /** Font */
     .menu_font              = "mono 12",
 
@@ -58,7 +58,7 @@ Settings config = {
     .terminal_emulator = "rofi-sensible-terminal",
     .ssh_client        = "ssh",
     /** Command when executing ssh. */
-    .ssh_command            = "{terminal} -e {ssh-client} {host}",
+    .ssh_command            = "{terminal} -e {ssh-client} {host} [-p {port}]",
     /** Command when running */
     .run_command            = "{cmd}",
     /** Command used to list executable commands. empty -> internal */
@@ -66,9 +66,9 @@ Settings config = {
     /** Command executed when running application in terminal */
     .run_shell_command      = "{terminal} -e {cmd}",
     /** Command executed on accep-entry-custom for window modus */
-    .window_command         = "xkill -id {window}",
+    .window_command         = "wmctrl -i -R {window}",
     /** No default icon theme, we search Adwaita and gnome as fallback */
-    .drun_icon_theme        = NULL,
+    .icon_theme             = NULL,
     /**
      * Location of the window.
      * Enumeration indicating location or gravity of window.
@@ -82,25 +82,27 @@ Settings config = {
      */
     .location               = WL_CENTER,
     /** Padding between elements */
-    .padding                =                                   5,
+    .padding                = 5,
     /** Y offset */
-    .y_offset               =                                   0,
+    .y_offset               = 0,
     /** X offset */
-    .x_offset               =                                   0,
+    .x_offset               = 0,
     /** Always show config.menu_lines lines, even if less lines are available */
     .fixed_num_lines        = TRUE,
     /** Do not use history */
     .disable_history        = FALSE,
+    /** Programs ignored for history */
+    .ignored_prefixes       = "",
     /** Sort the displayed list */
     .sort                   = FALSE,
     /** Use levenshtein sorting when matching */
-    .levenshtein_sort       = FALSE,
+    .sorting_method         = SORT_NORMAL,
     /** Case sensitivity of the search */
     .case_sensitive         = FALSE,
     /** Cycle through in the element list */
     .cycle                  = TRUE,
     /** Height of an element in #chars */
-    .element_height         =                                   1,
+    .element_height         = 1,
     /** Sidebar mode, show the modi */
     .sidebar_mode           = FALSE,
     /** auto select */
@@ -116,13 +118,18 @@ Settings config = {
     .matching_method = MM_NORMAL,
     /** Desktop entry fields to match*/
     .drun_match_fields      = "name,generic,exec,categories",
+    /**  Desktop format display */
+    .drun_display_format  = "{name} [<span weight='light' size='small'><i>({generic})</i></span>]",
+    /** Desktop entry show actions */
+    .drun_show_actions      = FALSE,
+    /** Desktop entry show actions */
     /** Window fields to match in window mode*/
     .window_match_fields    = "all",
     /** Monitor */
     .monitor                = "-5",
     /** set line margin */
-    .line_margin  =                                   2,
-    .line_padding =                                   1,
+    .line_margin  = 2,
+    .line_padding = 1,
     /** Set filter */
     .filter                 = NULL,
     /** Separator style: dash/solid */
@@ -131,12 +138,12 @@ Settings config = {
     .hide_scrollbar         = FALSE,
     .fullscreen             = FALSE,
     .fake_transparency      = FALSE,
-    .dpi                    =                                  -1,
-    .threads                =                                   0,
-    .scroll_method          =                                   0,
-    .scrollbar_width        =                                   8,
+    .dpi                    = -1,
+    .threads                = 0,
+    .scroll_method          = 0,
+    .scrollbar_width        = 8,
     .fake_background        = "screenshot",
-    .window_format          = "{w}    {i}{c}   {t}",
+    .window_format          = "{w}    {c}   {t}",
     .click_to_exit          = TRUE,
     .show_match             = TRUE,
     .theme                  = NULL,
@@ -145,6 +152,10 @@ Settings config = {
     .color_urgent           = NULL,
     .color_window           = NULL,
     .plugin_path            = PLUGIN_PATH,
-    .max_history_size       =                                  25,
+    .max_history_size       = 25,
     .combi_hide_mode_prefix = FALSE,
+
+    .matching_negate_char   = '-',
+
+    .cache_dir              = NULL,
 };
