@@ -36,6 +36,9 @@
  * @{
  */
 
+/**
+ * List of all scopes the mouse can interact on.
+ */
 typedef enum
 {
     SCOPE_GLOBAL,
@@ -45,8 +48,8 @@ typedef enum
 #define SCOPE_MIN_FIXED    SCOPE_MOUSE_EDITBOX
     SCOPE_MOUSE_EDITBOX,
     SCOPE_MOUSE_SCROLLBAR,
-    SCOPE_MOUSE_SIDEBAR_MODI,
-#define SCOPE_MAX_FIXED    SCOPE_MOUSE_SIDEBAR_MODI
+    SCOPE_MOUSE_MODE_SWITCHER,
+#define SCOPE_MAX_FIXED    SCOPE_MOUSE_MODE_SWITCHER
 } BindingsScope;
 
 /**
@@ -123,6 +126,7 @@ typedef enum
     CUSTOM_18,
     CUSTOM_19,
     SCREENSHOT,
+    CHANGE_ELLIPSIZE,
     TOGGLE_SORT,
     SELECT_ELEMENT_1,
     SELECT_ELEMENT_2,
@@ -136,6 +140,9 @@ typedef enum
     SELECT_ELEMENT_10,
 } KeyBindingAction;
 
+/**
+ * Actions mouse can take on the ListView.
+ */
 typedef enum
 {
     SCROLL_LEFT = 1,
@@ -144,6 +151,9 @@ typedef enum
     SCROLL_UP,
 } MouseBindingListviewAction;
 
+/**
+ * Actions mouse can take on the ListView element.
+ */
 typedef enum
 {
     SELECT_HOVERED_ENTRY = 1,
@@ -151,6 +161,9 @@ typedef enum
     ACCEPT_HOVERED_CUSTOM,
 } MouseBindingListviewElementAction;
 
+/**
+ * Default mouse actions.
+ */
 typedef enum
 {
     MOUSE_CLICK_DOWN = 1,
