@@ -68,6 +68,7 @@ typedef struct
     double           xalign;
 
     PangoFontMetrics *metrics;
+    PangoEllipsizeMode emode;
     int              left_offset;
     //
     const char       *theme_name;
@@ -78,14 +79,14 @@ typedef struct
  */
 typedef enum
 {
-    TB_AUTOHEIGHT    = 1 << 0,
-        TB_AUTOWIDTH = 1 << 1,
-        TB_EDITABLE  = 1 << 19,
-        TB_MARKUP    = 1 << 20,
-        TB_WRAP      = 1 << 21,
-        TB_PASSWORD  = 1 << 22,
-        TB_INDICATOR = 1 << 23,
-        TB_ICON      = 1 << 24,
+    TB_AUTOHEIGHT = 1 << 0,
+    TB_AUTOWIDTH  = 1 << 1,
+    TB_EDITABLE   = 1 << 19,
+    TB_MARKUP     = 1 << 20,
+    TB_WRAP       = 1 << 21,
+    TB_PASSWORD   = 1 << 22,
+    TB_INDICATOR  = 1 << 23,
+    TB_ICON       = 1 << 24,
 } TextboxFlags;
 /**
  * Flags indicating current state of the textbox.
@@ -319,6 +320,13 @@ PangoAttrList *textbox_get_pango_attributes ( textbox *tb );
  * @returns the visible text.
  */
 const char *textbox_get_visible_text ( const textbox *tb );
+/**
+ * @param wid The handle to the textbox.
+ *
+ * TODO: is this deprecated by widget::get_desired_width
+ *
+ * @returns the desired width of the textbox.
+ */
 int textbox_get_desired_width ( widget *wid );
 
 /**
@@ -327,5 +335,13 @@ int textbox_get_desired_width ( widget *wid );
  * Move the cursor to the end of the string.
  */
 void textbox_cursor_end ( textbox *tb );
+
+/**
+ * @param tb  Handle to the textbox
+ * @param mode The PangoEllipsizeMode to use displaying the text in the textbox
+ *
+ * Set the ellipsizing mode used on the string.
+ */
+void textbox_set_ellipsize ( textbox *tb, PangoEllipsizeMode mode );
 /*@}*/
 #endif //ROFI_TEXTBOX_H
