@@ -25,6 +25,7 @@
  *
  */
 
+/** The log domain of this dialog. */
 #define G_LOG_DOMAIN    "Dialogs.DRun"
 
 #include <config.h>
@@ -50,29 +51,35 @@
 #include "widgets/textbox.h"
 #include "history.h"
 #include "dialogs/drun.h"
-#include "nkutils-xdg-theme.h"
 #include "xcb.h"
 
+#include "rofi-icon-fetcher.h"
+
 #define DRUN_CACHE_FILE    "rofi3.druncache"
 
-#define DRUN_GROUP_NAME    "Desktop Entry"
+char *DRUN_GROUP_NAME = "Desktop Entry";
 
+typedef struct _DRunModePrivateData DRunModePrivateData;
 /**
  * Store extra information about the entry.
  * Currently the executable and if it should run in terminal.
  */
 typedef struct
 {
+    thread_state        st;
+    DRunModePrivateData *pd;
+    /* category */
+    char                *action;
     /* Root */
-    char *root;
+    char                *root;
     /* Path to desktop file */
-    char *path;
+    char                *path;
     /* Application id (.desktop filename) */
-    char *app_id;
+    char                *app_id;
     /* Desktop id */
-    char *desktop_id;
+    char                *desktop_id;
     /* Icon stuff */
-    char *icon_name;
+    char                *icon_name;
     /* Icon size is used to indicate what size is requested by the gui.
      * secondary it indicates if the request for a lookup has been issued (0 not issued )
      */
@@ -93,6 +100,8 @@ typedef struct
     GKeyFile        *key_file;
 
     gint            sort_index;
+
+    uint32_t        icon_fetch_uid;
 } DRunModeEntry;
 
 typedef struct
@@ -119,24 +128,21 @@ static DRunEntryField matching_entry_fields[DRUN_MATCH_NUM_FIELDS] = {
     { .entry_field_name = "comment",    .enabled = FALSE, }
 };
 
-typedef struct
+struct _DRunModePrivateData
 {
-    NkXdgThemeContext *xdg_context;
-    DRunModeEntry     *entry_list;
-    unsigned int      cmd_list_length;
-    unsigned int      cmd_list_length_actual;
+    DRunModeEntry *entry_list;
+    unsigned int  cmd_list_length;
+    unsigned int  cmd_list_length_actual;
     // List of disabled entries.
-    GHashTable        *disabled_entries;
-    unsigned int      disabled_entries_length;
-    GThreadPool       *pool;
-    unsigned int      expected_line_height;
-    DRunModeEntry     quit_entry;
+    GHashTable    *disabled_entries;
+    unsigned int  disabled_entries_length;
+    unsigned int  expected_line_height;
 
     // Theme
-    const gchar       *icon_theme;
+    const gchar   *icon_theme;
     // DE
-    gchar             **current_desktop_list;
-} DRunModePrivateData;
+    gchar         **current_desktop_list;
+};
 
 struct RegexEvalArg
 {
@@ -221,7 +227,7 @@ static void exec_cmd_entry ( DRunModeEntry *e )
     }
 
     const gchar *fp        = g_strstrip ( str );
-    gchar       *exec_path = g_key_file_get_string ( e->key_file, DRUN_GROUP_NAME, "Path", NULL );
+    gchar       *exec_path = g_key_file_get_string ( e->key_file, e->action, "Path", NULL );
     if ( exec_path != NULL && strlen ( exec_path ) == 0 ) {
         // If it is empty, ignore this property. (#529)
         g_free ( exec_path );
@@ -233,14 +239,14 @@ static void exec_cmd_entry ( DRunModeEntry *e )
         .icon   = e->icon_name,
         .app_id = e->app_id,
     };
-    gboolean                 sn       = g_key_file_get_boolean ( e->key_file, DRUN_GROUP_NAME, "StartupNotify", NULL );
+    gboolean                 sn       = g_key_file_get_boolean ( e->key_file, e->action, "StartupNotify", NULL );
     gchar                    *wmclass = NULL;
-    if ( sn && g_key_file_has_key ( e->key_file, DRUN_GROUP_NAME, "StartupWMClass", NULL ) ) {
-        context.wmclass = wmclass = g_key_file_get_string ( e->key_file, DRUN_GROUP_NAME, "StartupWMClass", NULL );
+    if ( sn && g_key_file_has_key ( e->key_file, e->action, "StartupWMClass", NULL ) ) {
+        context.wmclass = wmclass = g_key_file_get_string ( e->key_file, e->action, "StartupWMClass", NULL );
     }
 
     // Returns false if not found, if key not found, we don't want run in terminal.
-    gboolean terminal = g_key_file_get_boolean ( e->key_file, DRUN_GROUP_NAME, "Terminal", NULL );
+    gboolean terminal = g_key_file_get_boolean ( e->key_file, e->action, "Terminal", NULL );
     if ( helper_execute_command ( exec_path, fp, terminal, sn ? &context : NULL ) ) {
         char *path = g_build_filename ( cache_dir, DRUN_CACHE_FILE, NULL );
         // Store it based on the unique identifiers (desktop_id).
@@ -254,8 +260,9 @@ static void exec_cmd_entry ( DRunModeEntry *e )
 /**
  * This function absorbs/freeś path, so this is no longer available afterwards.
  */
-static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, const char *path, const gchar *basename )
+static void read_desktop_file ( DRunModePrivateData *pd, const char *root, const char *path, const gchar *basename, const char *action )
 {
+    int parse_action = ( config.drun_show_actions && action != DRUN_GROUP_NAME );
     // Create ID on stack.
     // We know strlen (path ) > strlen(root)+1
     const ssize_t id_len = strlen ( path ) - strlen ( root );
@@ -268,9 +275,9 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
     }
 
     // Check if item is on disabled list.
-    if ( g_hash_table_contains ( pd->disabled_entries, id ) ) {
+    if ( g_hash_table_contains ( pd->disabled_entries, id ) && !parse_action ) {
         g_debug ( "[%s] [%s] Skipping, was previously seen.", id, path );
-        return TRUE;
+        return ;
     }
     GKeyFile *kf    = g_key_file_new ();
     GError   *error = NULL;
@@ -280,7 +287,14 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
         g_debug ( "[%s] [%s] Failed to parse desktop file because: %s.", id, path, error->message );
         g_error_free ( error );
         g_key_file_free ( kf );
-        return FALSE;
+        return ;
+    }
+
+    if ( g_key_file_has_group ( kf, action ) == FALSE ) {
+        // No type? ignore.
+        g_debug ( "[%s] [%s] Invalid desktop file: No %s group", id, path, action );
+        g_key_file_free ( kf );
+        return ;
     }
     // Skip non Application entries.
     gchar *key = g_key_file_get_string ( kf, DRUN_GROUP_NAME, "Type", NULL );
@@ -288,13 +302,13 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
         // No type? ignore.
         g_debug ( "[%s] [%s] Invalid desktop file: No type indicated", id, path );
         g_key_file_free ( kf );
-        return FALSE;
+        return ;
     }
     if ( g_strcmp0 ( key, "Application" ) ) {
         g_debug ( "[%s] [%s] Skipping desktop file: Not of type application (%s)", id, path, key );
         g_free ( key );
         g_key_file_free ( kf );
-        return FALSE;
+        return ;
     }
     g_free ( key );
 
@@ -302,7 +316,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
     if ( !g_key_file_has_key ( kf, DRUN_GROUP_NAME, "Name", NULL ) ) {
         g_debug ( "[%s] [%s] Invalid desktop file: no 'Name' key present.", id, path );
         g_key_file_free ( kf );
-        return FALSE;
+        return ;
     }
 
     // Skip hidden entries.
@@ -310,7 +324,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
         g_debug ( "[%s] [%s] Adding desktop file to disabled list: 'Hidden' key is true", id, path );
         g_key_file_free ( kf );
         g_hash_table_add ( pd->disabled_entries, g_strdup ( id ) );
-        return FALSE;
+        return ;
     }
     if ( pd->current_desktop_list ) {
         gboolean show = TRUE;
@@ -345,7 +359,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
             g_debug ( "[%s] [%s] Adding desktop file to disabled list: 'OnlyShowIn'/'NotShowIn' keys don't match current desktop", id, path );
             g_key_file_free ( kf );
             g_hash_table_add ( pd->disabled_entries, g_strdup ( id ) );
-            return FALSE;
+            return ;
         }
     }
     // Skip entries that have NoDisplay set.
@@ -353,13 +367,13 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
         g_debug ( "[%s] [%s] Adding desktop file to disabled list: 'NoDisplay' key is true", id, path );
         g_key_file_free ( kf );
         g_hash_table_add ( pd->disabled_entries, g_strdup ( id ) );
-        return FALSE;
+        return ;
     }
     // We need Exec, don't support DBusActivatable
     if ( !g_key_file_has_key ( kf, DRUN_GROUP_NAME, "Exec", NULL ) ) {
         g_debug ( "[%s] [%s] Unsupported desktop file: no 'Exec' key present.", id, path );
         g_key_file_free ( kf );
-        return FALSE;
+        return ;
     }
 
     if ( g_key_file_has_key ( kf, DRUN_GROUP_NAME, "TryExec", NULL ) ) {
@@ -369,7 +383,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
             if ( fp == NULL ) {
                 g_free ( te );
                 g_key_file_free ( kf );
-                return FALSE;
+                return ;
             }
             g_free ( fp );
         }
@@ -377,7 +391,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
             if ( g_file_test ( te, G_FILE_TEST_IS_EXECUTABLE ) == FALSE ) {
                 g_free ( te );
                 g_key_file_free ( kf );
-                return FALSE;
+                return ;
             }
         }
         g_free ( te );
@@ -395,15 +409,24 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
         pd->entry_list[pd->cmd_list_length].sort_index = INT_MIN;
     }
     else {
-        pd->entry_list[pd->cmd_list_length].sort_index = -pd->cmd_list_length;
+        pd->entry_list[pd->cmd_list_length].sort_index = -nl;
     }
-    pd->entry_list[pd->cmd_list_length].icon_size  = 0;
-    pd->entry_list[pd->cmd_list_length].root       = g_strdup ( root );
-    pd->entry_list[pd->cmd_list_length].path       = g_strdup ( path );
-    pd->entry_list[pd->cmd_list_length].desktop_id = g_strdup ( id );
-    pd->entry_list[pd->cmd_list_length].app_id     = g_strndup ( basename, strlen ( basename ) - strlen ( ".desktop" ) );
+    pd->entry_list[pd->cmd_list_length].icon_size      = 0;
+    pd->entry_list[pd->cmd_list_length].icon_fetch_uid = 0;
+    pd->entry_list[pd->cmd_list_length].root           = g_strdup ( root );
+    pd->entry_list[pd->cmd_list_length].path           = g_strdup ( path );
+    pd->entry_list[pd->cmd_list_length].desktop_id     = g_strdup ( id );
+    pd->entry_list[pd->cmd_list_length].app_id         = g_strndup ( basename, strlen ( basename ) - strlen ( ".desktop" ) );
     gchar *n = g_key_file_get_locale_string ( kf, DRUN_GROUP_NAME, "Name", NULL, NULL );
-    pd->entry_list[pd->cmd_list_length].name = n;
+
+    if ( action != DRUN_GROUP_NAME ) {
+        gchar *na = g_key_file_get_locale_string ( kf, action, "Name", NULL, NULL );
+        gchar *l  = g_strdup_printf ( "%s - %s", n, na );
+        g_free ( n );
+        n = l;
+    }
+    pd->entry_list[pd->cmd_list_length].name   = n;
+    pd->entry_list[pd->cmd_list_length].action = DRUN_GROUP_NAME;
     gchar *gn = g_key_file_get_locale_string ( kf, DRUN_GROUP_NAME, "GenericName", NULL, NULL );
     pd->entry_list[pd->cmd_list_length].generic_name = gn;
     if ( matching_entry_fields[DRUN_MATCH_FIELD_CATEGORIES].enabled ) {
@@ -412,7 +435,7 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
     else {
         pd->entry_list[pd->cmd_list_length].categories = NULL;
     }
-    pd->entry_list[pd->cmd_list_length].exec = g_key_file_get_string ( kf, DRUN_GROUP_NAME, "Exec", NULL );
+    pd->entry_list[pd->cmd_list_length].exec = g_key_file_get_string ( kf, action, "Exec", NULL );
 
     if ( matching_entry_fields[DRUN_MATCH_FIELD_COMMENT].enabled ) {
         pd->entry_list[pd->cmd_list_length].comment = g_key_file_get_locale_string ( kf,
@@ -435,7 +458,18 @@ static gboolean read_desktop_file ( DRunModePrivateData *pd, const char *root, c
     g_hash_table_add ( pd->disabled_entries, g_strdup ( id ) );
     g_debug ( "[%s] Using file %s.", id, path );
     ( pd->cmd_list_length )++;
-    return TRUE;
+
+    if ( !parse_action ) {
+        gsize actions_length = 0;
+        char  **actions      = g_key_file_get_string_list ( kf, DRUN_GROUP_NAME, "Actions", &actions_length, NULL );
+        for ( gsize iter = 0; iter < actions_length; iter++ ) {
+            char *new_action = g_strdup_printf ( "Desktop Action %s", actions[iter] );
+            read_desktop_file ( pd, root, path, basename, new_action );
+            g_free ( new_action );
+        }
+        g_strfreev ( actions );
+    }
+    return ;
 }
 
 /**
@@ -489,7 +523,7 @@ static void walk_dir ( DRunModePrivateData *pd, const char *root, const char *di
         case DT_REG:
             // Skip files not ending on .desktop.
             if ( g_str_has_suffix ( file->d_name, ".desktop" ) ) {
-                read_desktop_file ( pd, root, filename, file->d_name );
+                read_desktop_file ( pd, root, filename, file->d_name, DRUN_GROUP_NAME );
             }
             break;
         case DT_DIR:
@@ -544,7 +578,11 @@ static gint drun_int_sort_list ( gconstpointer a, gconstpointer b, G_GNUC_UNUSED
     DRunModeEntry *da = (DRunModeEntry *) a;
     DRunModeEntry *db = (DRunModeEntry *) b;
 
-    return db->sort_index - da->sort_index;
+    if ( da->sort_index < 0 && db->sort_index < 0 ) {
+        return g_utf8_collate ( da->name, db->name );
+    } else {
+        return db->sort_index - da->sort_index;
+    }
 }
 
 static void get_apps ( DRunModePrivateData *pd )
@@ -582,60 +620,6 @@ static void get_apps ( DRunModePrivateData *pd )
     TICK_N ( "Sorting done." );
 }
 
-static void drun_icon_fetch ( gpointer data, gpointer user_data )
-{
-    g_debug ( "Starting up icon fetching thread." );
-    // as long as dr->icon is updated atomicly.. (is a pointer write atomic?)
-    // this should be fine running in another thread.
-    DRunModePrivateData *pd        = (DRunModePrivateData *) user_data;
-    DRunModeEntry       *dr        = (DRunModeEntry *) data;
-    const gchar         *themes[2] = {
-        config.drun_icon_theme,
-        NULL
-    };
-
-    if ( dr->icon_name == NULL ) {
-        return;
-    }
-    const gchar *icon_path;
-    gchar       *icon_path_ = NULL;
-
-    if ( g_path_is_absolute ( dr->icon_name ) ) {
-        icon_path = dr->icon_name;
-    }
-    else {
-        icon_path = icon_path_ = nk_xdg_theme_get_icon ( pd->xdg_context, themes, NULL, dr->icon_name, dr->icon_size, 1, TRUE );
-        if ( icon_path_ == NULL ) {
-            g_debug ( "Failed to get Icon %s(%d): n/a", dr->icon_name, dr->icon_size  );
-            return;
-        }
-        else{
-            g_debug ( "Found Icon %s(%d): %s", dr->icon_name, dr->icon_size, icon_path  );
-        }
-    }
-    cairo_surface_t *icon_surf = NULL;
-    if ( g_str_has_suffix ( icon_path, ".png" ) ) {
-        icon_surf = cairo_image_surface_create_from_png ( icon_path );
-    }
-    else if ( g_str_has_suffix ( icon_path, ".svg" ) ) {
-        icon_surf = cairo_image_surface_create_from_svg ( icon_path, dr->icon_size );
-    }
-    else {
-        g_debug ( "Icon type not yet supported: %s", icon_path  );
-    }
-    if ( icon_surf ) {
-        // Check if surface is valid.
-        if ( cairo_surface_status ( icon_surf ) != CAIRO_STATUS_SUCCESS ) {
-            g_debug ( "Icon failed to open: %s(%d): %s", dr->icon_name, dr->icon_size, icon_path );
-            cairo_surface_destroy ( icon_surf );
-            icon_surf = NULL;
-        }
-        dr->icon = icon_surf;
-    }
-    g_free ( icon_path_ );
-    rofi_view_reload ();
-}
-
 static void drun_mode_parse_entry_fields ()
 {
     char               *savept = NULL;
@@ -677,26 +661,13 @@ static int drun_mode_init ( Mode *sw )
     if ( mode_get_private_data ( sw ) != NULL ) {
         return TRUE;
     }
-
-    static const gchar * const drun_icon_fallback_themes[] = {
-        "Adwaita",
-        "gnome",
-        NULL
-    };
-    const gchar                *themes[2] = {
-        config.drun_icon_theme,
-        NULL
-    };
-    DRunModePrivateData        *pd = g_malloc0 ( sizeof ( *pd ) );
+    DRunModePrivateData *pd = g_malloc0 ( sizeof ( *pd ) );
     pd->disabled_entries = g_hash_table_new_full ( g_str_hash, g_str_equal, g_free, NULL );
     mode_set_private_data ( sw, (void *) pd );
     // current destkop
     const char *current_desktop = g_getenv ( "XDG_CURRENT_DESKTOP" );
     pd->current_desktop_list = current_desktop ? g_strsplit ( current_desktop, ":", 0 ) : NULL;
 
-    // Theme
-    pd->xdg_context = nk_xdg_theme_context_new ( drun_icon_fallback_themes, NULL );
-    nk_xdg_theme_preload_themes_icon ( pd->xdg_context, themes );
     drun_mode_parse_entry_fields ();
     get_apps ( pd );
     return TRUE;
@@ -715,6 +686,9 @@ static void drun_entry_clear ( DRunModeEntry *e )
     g_free ( e->name );
     g_free ( e->generic_name );
     g_free ( e->comment );
+    if ( e->action != DRUN_GROUP_NAME ) {
+        g_free ( e->action );
+    }
     g_strfreev ( e->categories );
     g_key_file_free ( e->key_file );
 }
@@ -746,10 +720,6 @@ static ModeMode drun_mode_result ( Mode *sw, int mretv, char **input, unsigned i
     else if ( ( mretv & MENU_ENTRY_DELETE ) && selected_line < rmpd->cmd_list_length ) {
         // Possitive sort index means it is in history.
         if ( rmpd->entry_list[selected_line].sort_index >= 0 ) {
-            if ( rmpd->pool ) {
-                g_thread_pool_free ( rmpd->pool, TRUE, TRUE );
-                rmpd->pool = NULL;
-            }
             delete_entry_history ( &( rmpd->entry_list[selected_line] ) );
             drun_entry_clear ( &( rmpd->entry_list[selected_line] ) );
             memmove ( &( rmpd->entry_list[selected_line] ), &rmpd->entry_list[selected_line + 1],
@@ -764,16 +734,11 @@ static void drun_mode_destroy ( Mode *sw )
 {
     DRunModePrivateData *rmpd = (DRunModePrivateData *) mode_get_private_data ( sw );
     if ( rmpd != NULL ) {
-        if ( rmpd->pool ) {
-            g_thread_pool_free ( rmpd->pool, TRUE, TRUE );
-            rmpd->pool = NULL;
-        }
         for ( size_t i = 0; i < rmpd->cmd_list_length; i++ ) {
             drun_entry_clear ( &( rmpd->entry_list[i] ) );
         }
         g_hash_table_destroy ( rmpd->disabled_entries );
         g_free ( rmpd->entry_list );
-        nk_xdg_theme_context_free ( rmpd->xdg_context );
 
         g_strfreev ( rmpd->current_desktop_list );
         g_free ( rmpd );
@@ -794,13 +759,41 @@ static char *_get_display_value ( const Mode *sw, unsigned int selected_line, in
     }
     /* Free temp storage. */
     DRunModeEntry *dr = &( pd->entry_list[selected_line] );
-    if ( dr->generic_name == NULL ) {
-        return g_markup_printf_escaped ( "%s", dr->name );
+    gchar *cats = NULL;
+    if ( dr->categories ){
+        char *tcats = g_strjoinv(",", dr->categories);
+        if ( tcats ) {
+            cats = g_markup_escape_text ( tcats, -1 );
+            g_free (tcats);
+        }
     }
-    else {
-        return g_markup_printf_escaped ( "%s <span weight='light' size='small'><i>(%s)</i></span>", dr->name,
-                                         dr->generic_name );
+    // Needed for display.
+    char *egn = NULL;
+    char *en = NULL;
+    char *ec = NULL;
+    if ( dr->generic_name ) {
+        egn = g_markup_escape_text ( dr->generic_name, -1 );
     }
+    if ( dr->name ) {
+        en = g_markup_escape_text ( dr->name, -1 );
+    }
+    if ( dr->comment ) {
+        ec = g_markup_escape_text ( dr->comment , -1 );
+    }
+
+
+    char *retv = helper_string_replace_if_exists ( config.drun_display_format,
+            "{generic}", egn,
+            "{name}", en,
+            "{comment}", ec,
+            "{exec}", dr->exec,
+            "{categories}", cats,
+            NULL);
+    g_free ( egn );
+    g_free ( en );
+    g_free ( ec );
+    g_free(cats);
+    return retv;
 }
 
 static cairo_surface_t *_get_icon ( const Mode *sw, unsigned int selected_line, int height )
@@ -808,16 +801,14 @@ static cairo_surface_t *_get_icon ( const Mode *sw, unsigned int selected_line,
     DRunModePrivateData *pd = (DRunModePrivateData *) mode_get_private_data ( sw );
     g_return_val_if_fail ( pd->entry_list != NULL, NULL );
     DRunModeEntry       *dr = &( pd->entry_list[selected_line] );
-    if ( pd->pool == NULL ) {
-        /* TODO: 4 threads good? */
-        pd->pool = g_thread_pool_new ( drun_icon_fetch, pd, 4, FALSE, NULL );
+    if ( dr->icon_name == NULL ) {
+        return NULL;
     }
-    if ( dr->icon_size == 0 ) {
-        dr->icon_size = height;
-        //g_async_queue_push ( pd->icon_fetch_queue, dr );
-        g_thread_pool_push ( pd->pool, dr, NULL );
+    if ( dr->icon_fetch_uid > 0 ) {
+        return rofi_icon_fetcher_get ( dr->icon_fetch_uid );
     }
-    return dr->icon;
+    dr->icon_fetch_uid = rofi_icon_fetcher_query ( dr->icon_name, height );
+    return rofi_icon_fetcher_get ( dr->icon_fetch_uid );
 }
 
 static char *drun_get_completion ( const Mode *sw, unsigned int index )
