@@ -26,6 +26,7 @@
  *
  */
 
+/** The log domain for this helper. */
 #define G_LOG_DOMAIN    "Helper"
 
 #include <config.h>
@@ -70,39 +71,14 @@ static int        stored_argc = 0;
 /** copy of the argv pointer for use in the commandline argument parser */
 static char       **stored_argv = NULL;
 
+char *helper_string_replace_if_exists_v ( char * string, GHashTable *h );
+
 void cmd_set_arguments ( int argc, char **argv )
 {
     stored_argc = argc;
     stored_argv = argv;
 }
 
-/**
- * @param info To Match information on.
- * @param res  The string being generated.
- * @param data User data
- *
- * Replace the entries. This function gets called by g_regex_replace_eval.
- *
- * @returns TRUE to stop replacement, FALSE to continue
- */
-static gboolean helper_eval_cb ( const GMatchInfo *info, GString *res, gpointer data )
-{
-    gchar *match;
-    // Get the match
-    match = g_match_info_fetch ( info, 0 );
-    if ( match != NULL ) {
-        // Lookup the match, so we can replace it.
-        gchar *r = g_hash_table_lookup ( (GHashTable *) data, match );
-        if ( r != NULL ) {
-            // Append the replacement to the string.
-            g_string_append ( res, r );
-        }
-        // Free match.
-        g_free ( match );
-    }
-    // Continue replacement.
-    return FALSE;
-}
 
 int helper_parse_setup ( char * string, char ***output, int *length, ... )
 {
@@ -128,11 +104,7 @@ int helper_parse_setup ( char * string, char ***output, int *length, ... )
     }
     va_end ( ap );
 
-    // Replace hits within {-\w+}.
-    GRegex *reg = g_regex_new ( "{[-\\w]+}", 0, 0, NULL );
-    char   *res = g_regex_replace_eval ( reg, string, -1, 0, 0, helper_eval_cb, h, NULL );
-    // Free regex.
-    g_regex_unref ( reg );
+    char *res = helper_string_replace_if_exists_v ( string, h );
     // Destroy key-value storage.
     g_hash_table_destroy ( h );
     // Parse the string into shell arguments.
@@ -220,7 +192,7 @@ static rofi_int_matcher * create_regex ( const char *input, int case_sensitive )
     GRegex           * retv = NULL;
     gchar            *r;
     rofi_int_matcher *rv = g_malloc0 ( sizeof ( rofi_int_matcher ) );
-    if ( input && input[0] == '-' ) {
+    if ( input && input[0] == config.matching_negate_char ) {
         rv->invert = 1;
         input++;
     }
@@ -577,6 +549,23 @@ int config_sanity_check ( void )
     GString *msg        = g_string_new (
         "<big><b>The configuration failed to validate:</b></big>\n" );
 
+    if ( config.sorting_method ) {
+        if ( g_strcmp0 ( config.sorting_method, "normal" ) == 0 ) {
+            config.sorting_method_enum = SORT_NORMAL;
+        }
+        else if ( g_strcmp0 ( config.sorting_method, "levenshtein" ) == 0 ) {
+            config.sorting_method_enum = SORT_NORMAL;
+        }
+        else if ( g_strcmp0 ( config.sorting_method, "fzf" ) == 0 ) {
+            config.sorting_method_enum = SORT_FZF;
+        }
+        else {
+            g_string_append_printf ( msg, "\t<b>config.sorting_method</b>=%s is not a valid sorting strategy.\nValid options are: normal or fzf.\n",
+                                     config.sorting_method );
+            found_error = 1;
+        }
+    }
+
     if ( config.matching ) {
         if ( g_strcmp0 ( config.matching, "regex" ) == 0 ) {
             config.matching_method = MM_REGEX;
@@ -1191,6 +1180,16 @@ void rofi_output_formatted_line ( const char *format, const char *string, int se
         else if ( format[i] == 's' ) {
             fputs ( string, stdout );
         }
+        else if ( format[i] == 'p' ) {
+            char *esc = NULL;
+            pango_parse_markup(string, -1, 0, NULL, &esc, NULL, NULL);
+            if ( esc ){
+                fputs ( esc, stdout );
+                g_free ( esc );
+            } else {
+                fputs ( "invalid string" , stdout );
+            }
+        }
         else if ( format[i] == 'q' ) {
             char *quote = g_shell_quote ( string );
             fputs ( quote, stdout );
@@ -1215,3 +1214,109 @@ void rofi_output_formatted_line ( const char *format, const char *string, int se
     fputc ( '\n', stdout );
     fflush ( stdout );
 }
+
+
+static gboolean helper_eval_cb2 ( const GMatchInfo *info, GString *res, gpointer data )
+{
+    gchar *match;
+    // Get the match
+    int num_match = g_match_info_get_match_count(info);
+    // Just {text} This is inside () 5.
+    if ( num_match == 5 ) {
+        match = g_match_info_fetch ( info, 4);
+        if ( match != NULL ) {
+            // Lookup the match, so we can replace it.
+            gchar *r = g_hash_table_lookup ( (GHashTable *) data, match );
+            if ( r != NULL ) {
+                // Append the replacement to the string.
+                g_string_append ( res, r );
+            }
+            // Free match.
+            g_free ( match );
+        }
+    }
+    // {} with [] guard around it.
+    else if ( num_match == 4 ) {
+        match = g_match_info_fetch ( info, 2);
+        if ( match != NULL ) {
+            // Lookup the match, so we can replace it.
+            gchar *r = g_hash_table_lookup ( (GHashTable *) data, match );
+            if ( r != NULL ) {
+                // Add (optional) prefix
+                gchar *prefix = g_match_info_fetch (info, 1);
+                g_string_append ( res, prefix );
+                g_free (prefix );
+                // Append the replacement to the string.
+                g_string_append ( res, r );
+                // Add (optional) postfix
+                gchar *post = g_match_info_fetch (info, 3);
+                g_string_append ( res, post );
+                g_free (post );
+            }
+            // Free match.
+            g_free ( match );
+        }
+    }
+    // Else we have an invalid match.
+    // Continue replacement.
+    return FALSE;
+}
+
+char *helper_string_replace_if_exists ( char * string, ... )
+{
+    GHashTable *h;
+    h = g_hash_table_new ( g_str_hash, g_str_equal );
+    va_list ap;
+    va_start ( ap, string );
+    // Add list from variable arguments.
+    while ( 1 ) {
+        char * key = va_arg ( ap, char * );
+        if ( key == (char *) 0 ) {
+            break;
+        }
+        char *value = va_arg ( ap, char * );
+        g_hash_table_insert ( h, key, value );
+    }
+    char *retv = helper_string_replace_if_exists_v ( string, h );
+    va_end ( ap );
+    // Destroy key-value storage.
+    g_hash_table_destroy ( h );
+    return retv;
+}
+/**
+ * @param string The string with elements to be replaced
+ * @param h      Hash table with set of {key}, value that will be replaced, terminated by  a NULL
+ *
+ * Items {key} are replaced by the value if '{key}' is passed as key/value pair, otherwise removed from string.
+ * If the {key} is in between []  all the text between [] are removed if {key}
+ * is not found. Otherwise key is replaced and [ & ] removed.
+ *
+ * This allows for optional replacement, f.e.   '{ssh-client} [-t  {title}] -e
+ * "{cmd}"' the '-t {title}' is only there if {title} is set.
+ *
+ * @returns a new string with the keys replaced.
+ */
+char *helper_string_replace_if_exists_v ( char * string, GHashTable *h )
+{
+    GError  *error = NULL;
+    char    *res   = NULL;
+
+    // Replace hits within {-\w+}.
+    GRegex *reg = g_regex_new ( "\\[(.*)({[-\\w]+})(.*)\\]|({[\\w-]+})", 0, 0, &error );
+    if ( error == NULL ){
+        res = g_regex_replace_eval ( reg, string, -1, 0, 0, helper_eval_cb2, h, &error );
+    }
+    // Free regex.
+    g_regex_unref ( reg );
+    // Throw error if shell parsing fails.
+    if ( error != NULL ) {
+        char *msg = g_strdup_printf ( "Failed to parse: '%s'\nError: '%s'", string, error->message );
+        rofi_view_error_dialog ( msg, FALSE );
+        g_free ( msg );
+        // print error.
+        g_error_free ( error );
+        g_free ( res );
+        return NULL;
+    }
+    return res;
+}
