@@ -25,6 +25,7 @@
  *
  */
 
+/** Log domain used by the theme engine.*/
 #define G_LOG_DOMAIN    "Theme"
 
 #include "config.h"
@@ -49,6 +50,11 @@ static gboolean distance_compare ( RofiDistance d, RofiDistance e )
     return d.type == e.type && d.distance == e.distance && d.style == e.style;
 }
 
+static gpointer rofi_g_list_strdup ( gconstpointer data, G_GNUC_UNUSED gpointer user_data )
+{
+    return g_strdup ( data );
+}
+
 ThemeWidget *rofi_theme_find_or_create_name ( ThemeWidget *base, const char *name )
 {
     for ( unsigned int i = 0; i < base->num_widgets; i++ ) {
@@ -85,11 +91,14 @@ Property* rofi_theme_property_copy ( Property *p )
         retv->value.s = g_strdup ( p->value.s );
         break;
     case P_LIST:
-        retv->value.list = g_list_copy_deep ( p->value.list, (GCopyFunc) g_strdup, NULL );
+        retv->value.list = g_list_copy_deep ( p->value.list, rofi_g_list_strdup, NULL );
         break;
     case P_LINK:
         retv->value.link.name = g_strdup ( p->value.link.name );
         retv->value.link.ref  = NULL;
+        if ( p->value.link.def_value ){
+            retv->value.link.def_value = rofi_theme_property_copy(p->value.link.def_value); 
+        }
         break;
     default:
         retv->value = p->value;
@@ -108,6 +117,9 @@ void rofi_theme_property_free ( Property *p )
     }
     else if ( p->type == P_LINK ) {
         g_free ( p->value.link.name );
+        if ( p->value.link.def_value ) {
+            rofi_theme_property_free ( p->value.link.def_value );
+        }
     }
     g_slice_free ( Property, p );
 }
@@ -175,103 +187,114 @@ const char * const WindowLocationStr[9] = {
     "west"
 };
 
+static void int_rofi_theme_print_property ( Property *p )
+{
+    switch ( p->type )
+    {
+        case P_LIST:
+            printf ( "[ " );
+            for ( GList *iter = p->value.list; iter != NULL; iter = g_list_next ( iter ) ) {
+                printf ( "%s", (char *) ( iter->data ) );
+                if ( iter->next != NULL ) {
+                    printf ( "," );
+                }
+            }
+            printf ( " ]" );
+            break;
+        case P_ORIENTATION:
+            printf ( "%s", ( p->value.i == ROFI_ORIENTATION_HORIZONTAL ) ? "horizontal" : "vertical" );
+            break;
+        case P_HIGHLIGHT:
+            if ( p->value.highlight.style & ROFI_HL_BOLD ) {
+                printf ( "bold " );
+            }
+            if ( p->value.highlight.style & ROFI_HL_UNDERLINE ) {
+                printf ( "underline " );
+            }
+            if ( p->value.highlight.style & ROFI_HL_STRIKETHROUGH ) {
+                printf ( "strikethrough " );
+            }
+            if ( p->value.highlight.style & ROFI_HL_ITALIC ) {
+                printf ( "italic " );
+            }
+            if ( p->value.highlight.style & ROFI_HL_COLOR ) {
+                printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
+                        ( p->value.highlight.color.red * 255.0 ),
+                        ( p->value.highlight.color.green * 255.0 ),
+                        ( p->value.highlight.color.blue * 255.0 ),
+                        ( p->value.highlight.color.alpha * 100.0 ) );
+            }
+            break;
+        case P_POSITION:
+            printf ( "%s", WindowLocationStr[p->value.i] );
+            break;
+        case P_STRING:
+            printf ( "\"%s\"", p->value.s );
+            break;
+        case P_INTEGER:
+            printf ( "%d", p->value.i );
+            break;
+        case P_DOUBLE:
+            printf ( "%.2f", p->value.f );
+            break;
+        case P_BOOLEAN:
+            printf ( "%s", p->value.b ? "true" : "false" );
+            break;
+        case P_COLOR:
+            printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
+                    ( p->value.color.red * 255.0 ),
+                    ( p->value.color.green * 255.0 ),
+                    ( p->value.color.blue * 255.0 ),
+                    ( p->value.color.alpha * 100.0 ) );
+            break;
+        case P_PADDING:
+            if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+                    distance_compare ( p->value.padding.left, p->value.padding.right ) &&
+                    distance_compare ( p->value.padding.left, p->value.padding.top ) ) {
+                rofi_theme_print_distance ( p->value.padding.left );
+            }
+            else if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+                    distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
+                rofi_theme_print_distance ( p->value.padding.top );
+                rofi_theme_print_distance ( p->value.padding.left );
+            }
+            else if ( !distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+                    distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
+                rofi_theme_print_distance ( p->value.padding.top );
+                rofi_theme_print_distance ( p->value.padding.left );
+                rofi_theme_print_distance ( p->value.padding.bottom );
+            }
+            else {
+                rofi_theme_print_distance ( p->value.padding.top );
+                rofi_theme_print_distance ( p->value.padding.right );
+                rofi_theme_print_distance ( p->value.padding.bottom );
+                rofi_theme_print_distance ( p->value.padding.left );
+            }
+            break;
+        case P_LINK:
+            if ( p->value.link.def_value) {
+                printf( "var( %s, ", p->value.link.name );
+                int_rofi_theme_print_property ( p->value.link.def_value );
+                printf (")");
+            }else {
+                printf ( "var(%s)", p->value.link.name );
+            }
+            break;
+        case P_INHERIT:
+            printf ( "inherit" );
+            break;
+        default:
+            break;
+    }
+
+}
+
 static void rofi_theme_print_property_index ( size_t pnl, int depth, Property *p )
 {
     int pl = strlen ( p->name );
     printf ( "%*s%s:%*s ", depth, "", p->name, (int) pnl - pl, "" );
-    switch ( p->type )
-    {
-    case P_LIST:
-        printf ( "[ " );
-        for ( GList *iter = p->value.list; iter != NULL; iter = g_list_next ( iter ) ) {
-            printf ( "%s", (char *) ( iter->data ) );
-            if ( iter->next != NULL ) {
-                printf ( "," );
-            }
-        }
-        printf ( " ];" );
-        break;
-    case P_ORIENTATION:
-        printf ( "%s;", ( p->value.i == ROFI_ORIENTATION_HORIZONTAL ) ? "horizontal" : "vertical" );
-        break;
-    case P_HIGHLIGHT:
-        if ( p->value.highlight.style & ROFI_HL_BOLD ) {
-            printf ( "bold " );
-        }
-        if ( p->value.highlight.style & ROFI_HL_UNDERLINE ) {
-            printf ( "underline " );
-        }
-        if ( p->value.highlight.style & ROFI_HL_STRIKETHROUGH ) {
-            printf ( "strikethrough " );
-        }
-        if ( p->value.highlight.style & ROFI_HL_ITALIC ) {
-            printf ( "italic " );
-        }
-        if ( p->value.highlight.style & ROFI_HL_COLOR ) {
-            printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
-                     ( p->value.highlight.color.red * 255.0 ),
-                     ( p->value.highlight.color.green * 255.0 ),
-                     ( p->value.highlight.color.blue * 255.0 ),
-                     ( p->value.highlight.color.alpha * 100.0 ) );
-        }
-        printf ( ";" );
-        break;
-    case P_POSITION:
-        printf ( "%s;", WindowLocationStr[p->value.i] );
-        break;
-    case P_STRING:
-        printf ( "\"%s\";", p->value.s );
-        break;
-    case P_INTEGER:
-        printf ( "%d;", p->value.i );
-        break;
-    case P_DOUBLE:
-        printf ( "%.2f;", p->value.f );
-        break;
-    case P_BOOLEAN:
-        printf ( "%s;", p->value.b ? "true" : "false" );
-        break;
-    case P_COLOR:
-        printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% );",
-                 ( p->value.color.red * 255.0 ),
-                 ( p->value.color.green * 255.0 ),
-                 ( p->value.color.blue * 255.0 ),
-                 ( p->value.color.alpha * 100.0 ) );
-        break;
-    case P_PADDING:
-        if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-             distance_compare ( p->value.padding.left, p->value.padding.right ) &&
-             distance_compare ( p->value.padding.left, p->value.padding.top ) ) {
-            rofi_theme_print_distance ( p->value.padding.left );
-        }
-        else if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-                  distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
-            rofi_theme_print_distance ( p->value.padding.top );
-            rofi_theme_print_distance ( p->value.padding.left );
-        }
-        else if ( !distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-                  distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
-            rofi_theme_print_distance ( p->value.padding.top );
-            rofi_theme_print_distance ( p->value.padding.left );
-            rofi_theme_print_distance ( p->value.padding.bottom );
-        }
-        else {
-            rofi_theme_print_distance ( p->value.padding.top );
-            rofi_theme_print_distance ( p->value.padding.right );
-            rofi_theme_print_distance ( p->value.padding.bottom );
-            rofi_theme_print_distance ( p->value.padding.left );
-        }
-        printf ( ";" );
-        break;
-    case P_LINK:
-        printf ( "%s;", p->value.link.name );
-        break;
-    case P_INHERIT:
-        printf ( "inherit;" );
-        break;
-    default:
-        break;
-    }
+    int_rofi_theme_print_property ( p );
+    putchar ( ';' );
     putchar ( '\n' );
 }
 
@@ -438,7 +461,8 @@ static ThemeWidget *rofi_theme_find ( ThemeWidget *widget, const char *name, con
 static void rofi_theme_resolve_link_property ( Property *p, int depth )
 {
     // Set name, remove '@' prefix.
-    const char *name = p->value.link.name + 1;
+    const char *name = p->value.link.name;// + (*(p->value.link.name)== '@'?1:0;
+    g_info ( "Resolving link to %s", p->value.link.name);
     if ( depth > 20 ) {
         g_warning ( "Found more then 20 redirects for property. Stopping." );
         p->value.link.ref = p;
@@ -447,6 +471,7 @@ static void rofi_theme_resolve_link_property ( Property *p, int depth )
 
     if ( rofi_theme->properties && g_hash_table_contains ( rofi_theme->properties, name ) ) {
         Property *pr = g_hash_table_lookup ( rofi_theme->properties, name );
+        g_info ("Resolving link %s found: %s", p->value.link.name, pr->name);
         if ( pr->type == P_LINK ) {
             if ( pr->value.link.ref == NULL ) {
                 rofi_theme_resolve_link_property ( pr, depth + 1 );
@@ -461,6 +486,11 @@ static void rofi_theme_resolve_link_property ( Property *p, int depth )
             return;
         }
     }
+    // No found and we have default value.
+    if ( p->value.link.def_value ){
+        p->value.link.ref = p->value.link.def_value;
+        return;
+    }
 
     // No found, set ref to self.
     p->value.link.ref = p;
@@ -631,7 +661,7 @@ double rofi_theme_get_double ( const widget *widget, const char *property, doubl
         return p->value.f;
     }
     // Fallback to integer if double is not found.
-    p   = rofi_theme_find_property ( wid, P_INTEGER, property, FALSE );
+    p = rofi_theme_find_property ( wid, P_INTEGER, property, FALSE );
     if ( p ) {
         if ( p->type == P_INHERIT ) {
             if ( widget->parent ) {
@@ -639,7 +669,7 @@ double rofi_theme_get_double ( const widget *widget, const char *property, doubl
             }
             return def;
         }
-        return (double)p->value.i;
+        return (double) p->value.i;
     }
     g_debug ( "Theme entry: #%s %s property %s unset.", widget->name, widget->state ? widget->state : "", property );
     return def;
@@ -700,7 +730,7 @@ GList *rofi_theme_get_list ( const widget *widget, const char * property, const
             }
         }
         else if ( p->type == P_LIST ) {
-            return g_list_copy_deep ( p->value.list, (GCopyFunc) g_strdup, NULL );
+            return g_list_copy_deep ( p->value.list, rofi_g_list_strdup, NULL );
         }
     }
     char **r = defaults ? g_strsplit ( defaults, ",", 0 ) : NULL;
@@ -872,7 +902,7 @@ void rofi_theme_convert_old ( void )
         if ( g_strcmp0 ( config.separator_style, "none" ) == 0 ) {
             const char *const str = "#listview { border: 0px; }";
             rofi_theme_parse_string ( str );
-            const char *const str2 = "#sidebar { border: 0px; }";
+            const char *const str2 = "#mode-switcher { border: 0px; }";
             rofi_theme_parse_string ( str2 );
             const char *const str3 = "#message { border: 0px; }";
             rofi_theme_parse_string ( str3 );
@@ -880,7 +910,7 @@ void rofi_theme_convert_old ( void )
         else if  ( g_strcmp0 ( config.separator_style, "solid" ) == 0 ) {
             const char *const str = "#listview { border: 2px solid 0px 0px 0px; }";
             rofi_theme_parse_string ( str );
-            const char *const str2 = "#sidebar { border: 2px solid 0px 0px 0px; }";
+            const char *const str2 = "#mode-switcher { border: 2px solid 0px 0px 0px; }";
             rofi_theme_parse_string ( str2 );
             const char *const str3 = "#message { border: 2px solid 0px 0px 0px; }";
             rofi_theme_parse_string ( str3 );
