@@ -35,14 +35,23 @@
 #include "settings.h"
 #include "theme.h"
 
+/** Default spacing between the elements in the listview. */
 #define DEFAULT_SPACING    2
 
+/**
+ * Orientation of the listview
+ */
 typedef enum
 {
+    /** Vertical (classical) list */
     LISTVIEW = ROFI_ORIENTATION_VERTICAL,
+    /** Horizontal list. (barview) */
     BARVIEW  = ROFI_ORIENTATION_HORIZONTAL,
 } ViewType;
 
+/**
+ * The moving direction of the selection, this (in barview) affects the scrolling.
+ */
 typedef enum
 {
     LEFT_TO_RIGHT = 0,
@@ -65,7 +74,6 @@ struct _listview
     unsigned int                selected;
 
     unsigned int                element_height;
-    unsigned int                element_width;
     unsigned int                max_rows;
     unsigned int                max_elements;
 
@@ -101,6 +109,8 @@ struct _listview
 
     char                        *listview_name;
 
+
+    PangoEllipsizeMode          emode;
     /** Barview */
     struct
     {
@@ -524,6 +534,7 @@ listview *listview_create ( widget *parent, const char *name, listview_update_ca
     lv->widget.get_desired_height = listview_get_desired_height;
     lv->eh                        = eh;
 
+    lv->emode     = PANGO_ELLIPSIZE_END;
     lv->scrollbar = scrollbar_create ( WIDGET ( lv ), "scrollbar" );
     // Calculate height of an element.
     //
@@ -822,3 +833,21 @@ void listview_set_fixed_num_lines ( listview *lv )
         lv->fixed_num_lines = TRUE;
     }
 }
+
+void listview_toggle_ellipsizing ( listview *lv )
+{
+    if ( lv ) {
+        PangoEllipsizeMode mode =  lv->emode;
+        if ( mode == PANGO_ELLIPSIZE_START ) { 
+            mode = PANGO_ELLIPSIZE_MIDDLE;
+        } else if ( mode == PANGO_ELLIPSIZE_MIDDLE ) {
+            mode = PANGO_ELLIPSIZE_END;
+        } else if ( mode == PANGO_ELLIPSIZE_END ) {
+            mode = PANGO_ELLIPSIZE_START;
+        }
+        lv->emode = mode;
+        for ( unsigned int i = 0; i < lv->cur_elements; i++ ) {
+            textbox_set_ellipsize ( lv->boxes[i], mode );
+        }
+    }
+}
