@@ -41,6 +41,7 @@
 
 #include "theme.h"
 
+/** The space reserved for the DOT when enabling multi-select. */
 #define DOT_OFFSET    15
 
 static void textbox_draw ( widget *, cairo_t * );
@@ -176,6 +177,7 @@ textbox* textbox_create ( widget *parent, WidgetType type, const char *name, Tex
     tb->widget.get_desired_height = textbox_get_desired_height;
     tb->widget.get_desired_width  = textbox_get_desired_width;
     tb->flags                     = flags;
+    tb->emode                     = PANGO_ELLIPSIZE_END;
 
     tb->changed = FALSE;
 
@@ -206,9 +208,9 @@ textbox* textbox_create ( widget *parent, WidgetType type, const char *name, Tex
         tb->widget.trigger_action = textbox_editable_trigger_action;
     }
 
-    tb->yalign = rofi_theme_get_double ( WIDGET ( tb ), "vertical-align", xalign );
+    tb->yalign = rofi_theme_get_double ( WIDGET ( tb ), "vertical-align", yalign );
     tb->yalign = MAX ( 0, MIN ( 1.0, tb->yalign ) );
-    tb->xalign = rofi_theme_get_double ( WIDGET ( tb ), "horizontal-align", yalign );
+    tb->xalign = rofi_theme_get_double ( WIDGET ( tb ), "horizontal-align", xalign );
     tb->xalign = MAX ( 0, MIN ( 1.0, tb->xalign ) );
 
     return tb;
@@ -334,6 +336,14 @@ void textbox_text ( textbox *tb, const char *text )
 
 void textbox_icon ( textbox *tb, cairo_surface_t *icon )
 {
+    // Add our reference to the surface.
+    if ( icon != NULL ) {
+        cairo_surface_reference ( icon );
+    }
+    if ( tb->icon ) {
+        // If we overwrite an old one, destroy the reference we hold.
+        cairo_surface_destroy ( tb->icon );
+    }
     tb->icon = icon;
 
     widget_queue_redraw ( WIDGET ( tb ) );
@@ -353,7 +363,9 @@ void textbox_moveresize ( textbox *tb, int x, int y, int w, int h )
             pango_layout_set_ellipsize ( tb->layout, PANGO_ELLIPSIZE_MIDDLE );
         }
         else if ( ( tb->flags & TB_WRAP ) != TB_WRAP ) {
-            pango_layout_set_ellipsize ( tb->layout, PANGO_ELLIPSIZE_END );
+            pango_layout_set_ellipsize ( tb->layout, tb->emode );
+        } else {
+            pango_layout_set_ellipsize ( tb->layout, PANGO_ELLIPSIZE_NONE );
         }
     }
 
@@ -390,6 +402,10 @@ static void textbox_free ( widget *wid )
     }
     g_free ( tb->text );
 
+    if ( tb->icon ) {
+        cairo_surface_destroy ( tb->icon );
+        tb->icon = NULL;
+    }
     if ( tb->layout != NULL ) {
         g_object_unref ( tb->layout );
     }
@@ -765,15 +781,15 @@ int textbox_keybinding ( textbox *tb, KeyBindingAction action )
     case REMOVE_CHAR_FORWARD:
         textbox_cursor_del ( tb );
         return 1;
-    // Alt-B
+    // Alt-B, Ctrl-Left
     case MOVE_WORD_BACK:
         textbox_cursor_dec_word ( tb );
         return 2;
-    // Alt-F
+    // Alt-F, Ctrl-Right
     case MOVE_WORD_FORWARD:
         textbox_cursor_inc_word ( tb );
         return 2;
-    // BackSpace, Ctrl-h
+    // BackSpace, Shift-BackSpace, Ctrl-h
     case REMOVE_CHAR_BACK:
         textbox_cursor_bkspc ( tb );
         return 1;
@@ -878,7 +894,7 @@ int textbox_get_font_width ( const textbox *tb )
     return rect.width + rect.x;
 }
 
-/** Caching for the expected character height. */
+/** Caching for the estimated character height. (em) */
 static double char_height = -1;
 double textbox_get_estimated_char_height ( void )
 {
@@ -900,6 +916,7 @@ double textbox_get_estimated_char_width ( void )
     return char_width;
 }
 
+/** Cache storing the estimated width of a digit (ch). */
 static double ch_width = -1;
 double textbox_get_estimated_ch ( void )
 {
@@ -936,3 +953,17 @@ int textbox_get_desired_width ( widget *wid )
     pango_layout_set_width ( tb->layout, old_width );
     return width + padding + offset;
 }
+
+
+void textbox_set_ellipsize ( textbox *tb, PangoEllipsizeMode mode )
+{
+    if ( tb )
+    {
+        tb->emode = mode;
+        if ( ( tb->flags & TB_WRAP ) != TB_WRAP ) {
+            // Store the mode.
+            pango_layout_set_ellipsize ( tb->layout, tb->emode );
+            widget_queue_redraw ( WIDGET ( tb ) );
+        }
+    }
+}
