@@ -130,17 +130,23 @@ static XrmOption xrmOptions[] = {
       "Command executed on accep-entry-custom for window modus", CONFIG_DEFAULT },
     { xrm_String,  "window-match-fields",    { .str  = &config.window_match_fields            }, NULL,
       "Window fields to match in window mode", CONFIG_DEFAULT },
-    { xrm_String,  "drun-icon-theme",        { .str  = &config.drun_icon_theme                }, NULL,
+    { xrm_String,  "icon-theme",             { .str  = &config.icon_theme                     }, NULL,
       "Theme to use to look for icons", CONFIG_DEFAULT },
 
     { xrm_String,  "drun-match-fields",      { .str  = &config.drun_match_fields              }, NULL,
       "Desktop entry fields to match in drun", CONFIG_DEFAULT },
+    { xrm_Boolean, "drun-show-actions",      { .num  = &config.drun_show_actions              }, NULL,
+      "Desktop entry show actions.", CONFIG_DEFAULT },
+    { xrm_String,  "drun-display-format",    { .str  = &config.drun_display_format            }, NULL,
+      "DRUN format string. (Supports: generic,name,comment,exec,categories)", CONFIG_DEFAULT },
     { xrm_Boolean, "disable-history",        { .num  = &config.disable_history                }, NULL,
       "Disable history in run/ssh", CONFIG_DEFAULT },
+    { xrm_String,  "ignored-prefixes",       { .str  = &config.ignored_prefixes               }, NULL,
+      "Programs ignored for history", CONFIG_DEFAULT },
     { xrm_Boolean, "sort",                   { .num  = &config.sort                           }, NULL,
       "Use sorting", CONFIG_DEFAULT },
-    { xrm_Boolean, "levenshtein-sort",       { .num  = &config.levenshtein_sort               }, NULL,
-      "Use levenshtein sorting also for fuzzy matching", CONFIG_DEFAULT },
+    { xrm_String,  "sorting-method",         { .str  = &config.sorting_method                 }, NULL,
+      "Choose the strategy used for sorting: normal (levenshtein) or fzf.", CONFIG_DEFAULT },
     { xrm_Boolean, "case-sensitive",         { .num  = &config.case_sensitive                 }, NULL,
       "Set case-sensitivity", CONFIG_DEFAULT },
     { xrm_Boolean, "cycle",                  { .num  = &config.cycle                          }, NULL,
@@ -189,9 +195,9 @@ static XrmOption xrmOptions[] = {
     { xrm_Number,  "scroll-method",          { .num  = &config.scroll_method                  }, NULL,
       "Scrolling method. (0: Page, 1: Centered)", CONFIG_DEFAULT },
     { xrm_String,  "fake-background",        { .str  = &config.fake_background                }, NULL,
-      "Background to use for fake transparency. (background or screenshot)", CONFIG_DEFAULT },
+      "Background to use for fake transparency. (background or screenshot) *DEPRECATED*", CONFIG_DEFAULT },
     { xrm_String,  "window-format",          { .str  = &config.window_format                  }, NULL,
-      "Window Format. w (desktop name), t (title), n (name), r (role), c (class) *DEPRECATED*", CONFIG_DEFAULT },
+      "Window Format. w (desktop name), t (title), n (name), r (role), c (class)", CONFIG_DEFAULT },
     { xrm_Boolean, "click-to-exit",          { .snum = &config.click_to_exit                  }, NULL,
       "Click outside the window to exit", CONFIG_DEFAULT },
     { xrm_Boolean, "show-match",             { .snum = &config.show_match                     }, NULL,
@@ -210,6 +216,10 @@ static XrmOption xrmOptions[] = {
       "Max history size (WARNING: can cause slowdowns when set to high).", CONFIG_DEFAULT },
     { xrm_Boolean, "combi-hide-mode-prefix", { .snum = &config.combi_hide_mode_prefix         }, NULL,
       "Hide the prefix mode prefix on the combi view.", CONFIG_DEFAULT },
+    { xrm_Char, "matching-negate-char",     { .charc= &config.matching_negate_char            }, NULL,
+      "Set the character used to negate the matching. ('\\0' to disable)", CONFIG_DEFAULT },
+    { xrm_String, "cache-dir",               { .str = &config.cache_dir                       }, NULL,
+      "Directory where history and temporary files are stored.", CONFIG_DEFAULT },
 };
 
 /** Dynamic array of extra options */
@@ -436,7 +446,7 @@ static gboolean __config_parser_set_property ( XrmOption *option, const Property
     }
     else if ( option->type == xrm_Number ) {
         if ( p->type != P_INTEGER ) {
-            *error = g_strdup_printf ( "Option: %s needs to be set with a numger not a %s.", option->name, PropertyTypeName[p->type] );
+            *error = g_strdup_printf ( "Option: %s needs to be set with a number not a %s.", option->name, PropertyTypeName[p->type] );
             return TRUE;
         }
         *( option->value.snum ) = p->value.i;
@@ -444,7 +454,7 @@ static gboolean __config_parser_set_property ( XrmOption *option, const Property
     }
     else if ( option->type == xrm_SNumber ) {
         if ( p->type != P_INTEGER ) {
-            *error = g_strdup_printf ( "Option: %s needs to be set with a numger not a %s.", option->name, PropertyTypeName[p->type] );
+            *error = g_strdup_printf ( "Option: %s needs to be set with a number not a %s.", option->name, PropertyTypeName[p->type] );
             return TRUE;
         }
         *( option->value.num ) = (unsigned int ) ( p->value.i );
