@@ -420,6 +420,18 @@ static const struct {
             .result = NULL,
         }
     },
+    {
+        .testpath = "/nkutils/xdg-theme/icon/theme/recursive",
+        .data = {
+            .type = TYPE_ICON,
+            .themes = { [0] = "recursive-theme-test" },
+            .name = "test-icon",
+            .size = 10,
+            .scale = 1,
+            .svg = TRUE,
+            .result = SRCDIR G_DIR_SEPARATOR_S "tests/icons/recursive-theme-test/test-dir/test-icon.svg",
+        }
+    },
     {
         .testpath = "/nkutils/xdg-theme/sound/found/variant",
         .data = {
@@ -502,7 +514,7 @@ main(int argc, char *argv[])
 
     g_setenv("HOME", "/var/empty", TRUE);
     g_setenv("XDG_CONFIG_HOME", SRCDIR G_DIR_SEPARATOR_S "tests", TRUE);
-    g_setenv("XDG_DATA_HOME", "/var/empty/.local/share", TRUE);
+    g_setenv("XDG_DATA_HOME", SRCDIR G_DIR_SEPARATOR_S "tests", TRUE);
     g_setenv("XDG_DATA_DIRS", "/usr/share/", TRUE);
     g_setenv("XDG_SESSION_DESKTOP", "", TRUE);
     g_setenv("XDG_CURRENT_DESKTOP", "", TRUE);
