@@ -1,3 +1,30 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include <assert.h>
 #include <locale.h>
 #include <glib.h>
@@ -5,12 +32,13 @@
 #include <helper.h>
 #include <string.h>
 #include <xcb/xcb_ewmh.h>
+#include "display.h"
+#include "xcb.h"
 #include "xcb-internal.h"
 #include "rofi.h"
 #include "settings.h"
 
 static int       test = 0;
-struct xcb_stuff *xcb;
 
 #define TASSERT( a )        {                            \
         assert ( a );                                    \
@@ -24,20 +52,25 @@ struct xcb_stuff *xcb;
             abort ( );                                                                   \
         }                                                                                \
 }
-
+void rofi_add_error_message ( G_GNUC_UNUSED GString *msg )
+{
+}
 int rofi_view_error_dialog ( const char *msg, G_GNUC_UNUSED int markup )
 {
     fputs ( msg, stderr );
     return TRUE;
 }
 
-int show_error_message ( const char *msg, int markup )
+int monitor_active ( G_GNUC_UNUSED workarea *mon )
 {
-    rofi_view_error_dialog ( msg, markup );
     return 0;
 }
 
-int main ( int argc, char ** argv )
+void display_startup_notification ( G_GNUC_UNUSED RofiHelperExecuteContext *context, G_GNUC_UNUSED GSpawnChildSetupFunc *child_setup, G_GNUC_UNUSED gpointer *user_data )
+{
+}
+
+int main ( G_GNUC_UNUSED int argc, G_GNUC_UNUSED char ** argv )
 {
 
     if ( setlocale ( LC_ALL, "" ) == NULL ) {
@@ -74,17 +107,17 @@ int main ( int argc, char ** argv )
     TASSERT ( str == list[3] );
 
     unsigned int u = 1234;
-    unsigned int i = -1234;
+    int d = -1234;
     TASSERT ( find_arg_uint ( "-x", &u ) == FALSE );
     TASSERT ( u == 1234 );
-    TASSERT ( find_arg_int ( "-x", &i ) == FALSE );
-    TASSERT ( i == -1234 );
+    TASSERT ( find_arg_int ( "-x", &d ) == FALSE );
+    TASSERT ( d == -1234 );
     TASSERT ( find_arg_uint ( "-u", &u ) == TRUE );
     TASSERT ( u == 4 );
     TASSERT ( find_arg_uint ( "-i", &u ) == TRUE );
     TASSERT ( u == 4294967293 );
-    TASSERT ( find_arg_int ( "-i", &i ) == TRUE );
-    TASSERT ( i == -3 );
+    TASSERT ( find_arg_int ( "-i", &d ) == TRUE );
+    TASSERT ( d == -3 );
 
     g_strfreev ( list );
 
