@@ -33,11 +33,11 @@
 #include <history.h>
 #include <string.h>
 
-static int test = 0;
+static unsigned int test = 0;
 
 #define TASSERT( a )    {                                \
         assert ( a );                                    \
-        printf ( "Test %i passed (%s)\n", ++test, # a ); \
+        printf ( "Test %u passed (%s)\n", ++test, # a ); \
 }
 
 const char *file = "text";
@@ -87,8 +87,8 @@ static void history_test ( void )
     g_strfreev ( retv );
 
     for ( unsigned int in = length + 1; in < 26; in++ ) {
-        char *p = g_strdup_printf ( "aap%i", in );
-        printf ( "%s- %d\n", p, in );
+        char *p = g_strdup_printf ( "aap%u", in );
+        printf ( "%s- %u\n", p, in );
         history_set ( file, p );
         retv = history_get_list ( file, &length );
 
